"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const CSP_1 = require("../models/CSP");
const crunchCSPHeaderValue_1 = __importDefault(require("./crunchCSPHeaderValue"));
function getCSPDirective(value, defaultValue, mergeDefaultDirectives = false) {
    // if user configured value is false, return early
    if (value === false) {
        return false;
    }
    // ensure any string values are split to enable removal of duplicates
    const valueArray = [value].flat().reduce((accumulator, current) => {
        if (typeof current === 'undefined') {
            return accumulator;
        }
        accumulator.push(...current.trim().split(/\s+/));
        return accumulator;
    }, []);
    // flatten default values
    const defaultValueArray = [defaultValue].flat().filter((v) => typeof v === 'string');
    // merge values with default if required
    const mergedValueArray = mergeDefaultDirectives
        ? [...defaultValueArray, ...valueArray]
        : valueArray;
    // de-duplicate merged values
    const uniqueValueArray = [...new Set(mergedValueArray)];
    // remove value "'none'" if the array contains other values
    const validValueArray = uniqueValueArray.length > 1
        ? uniqueValueArray.filter((v) => v !== "'none'")
        : uniqueValueArray;
    // only return user configured values if present, otherwise return default
    return validValueArray.length > 0 ? validValueArray : defaultValueArray;
}
function buildCSPHeaders(options = {}) {
    const { contentSecurityPolicy = {}, isDev, } = options;
    if (contentSecurityPolicy === false) {
        return [];
    }
    // ensure mergeDefaultDirectives option is a boolean, anything other than boolean `true` means `false`
    const mergeDefaultDirectives = contentSecurityPolicy.mergeDefaultDirectives === true;
    // Content Security Policy
    const directives = {};
    CSP_1.CSP_REQUIRED_DIRECTIVE_NAMES.forEach((name) => {
        directives[name] = getCSPDirective(contentSecurityPolicy[name], CSP_1.CSP_REQUIRED_DIRECTIVE_DEFAULTS[name], mergeDefaultDirectives);
    });
    CSP_1.CSP_OPTIONAL_DIRECTIVE_NAMES.forEach((name) => {
        const configuredValue = contentSecurityPolicy[name];
        if (typeof configuredValue !== 'undefined') {
            directives[name] = getCSPDirective(configuredValue);
        }
    });
    CSP_1.CSP_REPORT_DIRECTIVE_NAMES.some((name) => {
        const configuredValue = contentSecurityPolicy[name];
        if (typeof configuredValue !== 'undefined') {
            const normalisedValue = getCSPDirective(configuredValue);
            CSP_1.CSP_REPORT_DIRECTIVE_NAMES.forEach((name) => {
                directives[name] = normalisedValue;
            });
        }
    });
    CSP_1.CSP_DIRECTIVE_NAMES.forEach((name) => {
        const normalisedValue = directives[name];
        if (normalisedValue === false) {
            delete directives[name];
        }
    });
    if (isDev) {
        CSP_1.CSP_DEVELOPMENT_DIRECTIVE_NAMES.forEach((name) => {
            const normalisedValue = getCSPDirective(CSP_1.CSP_DEVELOPMENT_DIRECTIVES[name]);
            if (normalisedValue === false) {
                return;
            }
            const existingValue = directives[name];
            if (typeof existingValue === 'undefined' || existingValue === false) {
                directives[name] = normalisedValue;
            }
            else {
                directives[name] = [...new Set([...existingValue, ...normalisedValue])];
            }
        });
    }
    const cspString = (0, crunchCSPHeaderValue_1.default)(directives);
    const reportOnly = typeof contentSecurityPolicy.reportOnly === 'boolean' &&
        contentSecurityPolicy.reportOnly
        ? '-Report-Only'
        : '';
    const cspHeaderNames = [
        `Content-Security-Policy${reportOnly}`,
        `X-Content-Security-Policy${reportOnly}`,
        'X-WebKit-CSP',
    ];
    return cspHeaderNames.map((headerName) => ({
        key: headerName,
        value: cspString,
    }));
}
exports.default = buildCSPHeaders;
